/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

header {
  display: block;
}

#global-header {
  border-top: 2px solid #686868;
}

#global-header #mozilla-tab {
  background: url("../media/mozilla-tab.png") no-repeat scroll 0 0 transparent;
  height: 50px;
  position: absolute;
  right: 0;
  text-indent: -10000em;
  top: 0;
  width: 118px;
  z-index: 1;
}

#global-header .menu form {
  margin-top: 0.5em;
}

#global-header .menu {
  font-size: 12px;
  position: absolute;
  right: 118px;
  top: 0;
  z-index: 1;
}

#global-header .menu p {
  border-left: 1px solid #9Fa7aF;
  display: inline-block;
  height: 32px;
  line-height: 32px;
  padding: 0 1em;
}

#site-header {
  margin: 35px 0 0;
}

#site-header h1, #site-header #version, #site-header #site-nav {
  margin-left: 64px;
}

#site-header h1 {
  display: inline;
}

#site-header h1 a {
  -moz-transition-duration: 0.2s;
  -moz-transition-property: color, text-shadow;
  font-size: 40px;
  text-transform: uppercase;
  color: #333333;
}

#site-header, #site-header h1 a {
  display: inline-block;
  font-family: MetaBlack;
  font-weight: bold;
  letter-spacing: -1px;
  text-decoration: none;
}

#site-header h1 a span {
  background: url("../media/firefox-logo.png") no-repeat scroll 0 0 transparent;
  height: 62px;
  left: 0;
  position: absolute;
  top: 0;
  width: 54px;
}

#site-header h1 a:hover, #site-header h1 a:focus {
  color: #043b84;
  text-shadow: 0 0 40px #ffffff, 0 0 20px #ffffff, 0 0 10px #ffffff, 0 0 10px #ffffff, 0 0 10px #ffffff;
}

#site-header #site-search {
  display: none;
}
